#!/usr/bin/python -t
# vim: set fileencoding=UTF-8 :

import os
import tempfile
import unittest
import shutil
from xml.parsers.expat import ExpatError as ExpatError

# modules under test
import scheduler
import errors
import xmloutput

class Generator(object):
    payloadSkeleton = """
    [HEADER]
    NAME=\"%(name)s\"
    FILE_NAME=\"%(file_name)s\"
    DESCRIPTION=\"%(description)s\"
    LEVELS=\"%(levels)s\"
    GROUPS=\"%(groups)s\"
    DEPS=\"%(deps)s\"
    REQUIRES=\"%(requires)s\"
    NEED_DATA_DIR=%(data_dir)s

    [DEFAULT]
    PROM1="value1"
    PROM2="value2"

    [LEVEL2]
    PROM1="value1_level2"
    """

    defaults = { "name" : "test",
                 "file_name" : "script.sh",
                 "description" : "A test description",
                 "levels" : "1 2 3 4 5",
                 "groups" : "general",
                 "deps" : "bash find",
                 "requires" : "bash",
                 "data_dir" : 0
               }


    def __init__(self, fhandle, dir):
        self.fhandle = fhandle
        self.directory = dir

    def customizeDescription(**kwargs):
        all_params = Generator.defaults.copy()
        all_params.update(dict(kwargs))
        return Generator.payloadSkeleton % all_params
    customizeDescription= staticmethod(customizeDescription)

    def getDefaultPayload():
        return Generator.payloadSkeleton % Generator.defaults
    getDefaultPayload = staticmethod(getDefaultPayload)

    def injectPayload(self, payload):
        # deindent
        payload = "\n".join( [ line.lstrip() for line in payload.split("\n") if line != "" ] )
        self.fhandle.write(payload)
        self.fhandle.flush()

    def insertNamedFile(self, name, payload=None, executable=False):
        tmp_file_path = os.path.join(self.directory, name)
        self.fhandle = open(tmp_file_path, "w")
        
        if payload:
            self.injectPayload(payload)
        if executable:
            os.chmod(tmp_file_path,0755)

        self.fhandle.close()
        return tmp_file_path

    def createSimpleTestCase(self, path, dsc_name="test.dsc", expectPass=True):
        scriptPayload = """
        #!/bin/bash -x

        ls -ld %s >/dev/null 2>&1
        """

        self.directory = path
        dsc_path = self.insertNamedFile(name=dsc_name, payload=Generator.getDefaultPayload())

        scriptPayload = scriptPayload % (expectPass and "/tmp" or "/no/such/path")
        self.insertNamedFile(name=Generator.defaults["file_name"], executable=True, payload=scriptPayload)

        return dsc_path

    def createVariableTestCase(self, path, dsc_name, var, value):
        scriptPayload = """#!/bin/bash 

        if [ "$%s" == %s ]; then
            echo "Hint: 001 - Use the force, Luke"
            exit 0;
        fi

        echo "Error: 666 - Obi-Wan generic error"
        exit 1
        """

        self.directory = path
        dsc_path = self.insertNamedFile(name=dsc_name, payload=Generator.getDefaultPayload())

        scriptPayload = scriptPayload % (var, value)
        self.insertNamedFile(name=Generator.defaults["file_name"], executable=True, payload=scriptPayload)

        return dsc_path

class TestDescription(unittest.TestCase):
    def setUp(self):
        (self.dsc_handle, self.dsc_path) = tempfile.mkstemp(prefix="test-sectool-scheduler")
        self.dsc_file = os.fdopen(self.dsc_handle, "w")

        self.utils = Generator(self.dsc_file, None)

    def tearDown(self):
        self.dsc_file.close()
        os.remove(self.dsc_path)

    def testValidDescription(self):
        self.utils.injectPayload(payload=Generator.getDefaultPayload())
        dp = scheduler.Description(self.dsc_path)

    def testNotAnIniFormat(self):
        self.utils.injectPayload(payload="foo\nbar\nbaz\n")
        self.assertRaises(errors.ValidationError, scheduler.Description, self.dsc_path)

    def testMissingHeader(self):
        payload = """
        [DEFAULT]
        GRP="/dev/null"
        PERM="100664"
        UMASK="077
        """
        self.utils.injectPayload(payload)
        self.assertRaises(errors.ValidationError, scheduler.Description, self.dsc_path)

    def testMissingRequiredItem(self):
        payload = """
        [HEADER]
        NAME=umask
        DESCRIPTION=...
        LEVELS=1 2 3 4 5
        GROUP=account
        DEPS=awk lsof find
        """

        self.utils.injectPayload(payload)
        self.assertRaises(errors.ValidationError, scheduler.Description, self.dsc_path)

    def testInvalidLevels(self):
        payload = """
        [HEADER]
        NAME=umask
        DESCRIPTION=...
        LEVELS=1 2 a 4 5
        GROUP=account
        DEPS=awk lsof find
        """

        self.utils.injectPayload(payload)
        self.assertRaises(errors.ValidationError, scheduler.Description, self.dsc_path)

    def testInvalidName(self):
        payload  = """
        [HEADER]
        NAME="foo bar"
        DESCRIPTION=...
        LEVELS=1 2 3 4 5
        GROUP=account
        DEPS=awk lsof find
        """

        self.utils.injectPayload(payload)
        self.assertRaises(errors.ValidationError, scheduler.Description, self.dsc_path)

    def testGettingItems(self):
        self.utils.injectPayload(payload=Generator.getDefaultPayload())
        dsc = scheduler.Description(self.dsc_path)

        self.assertEqual(dsc["HEADER"]["DESCRIPTION"], "A test description")

    def testSectionRepr(self):
        self.utils.injectPayload(payload=Generator.getDefaultPayload())
        dsc = scheduler.Description(self.dsc_path)
        self.assert_(dsc["HEADER"])

    def testMultiLineDescription(self):
        """
        Multi-line descriptions must have the \n symbol escaped, otherwise we
        get a ParseError.
        """
        payload = """
        [HEADER]
        NAME="umask"
        FILE_NAME="script.sh"
        DESCRIPTION="This is a very long description.\
        It is realy looong. But it cannot span over several lines unless we want the parser to puke"
        LEVELS="1 2 3 4 5"
        GROUPS="account other"
        DEPS="awk lsof find"
        """

        self.utils.injectPayload(payload)
        dp = scheduler.Description(self.dsc_path)

    def testEmptyDescription(self):
        self.utils.injectPayload("")
        self.assertRaises(errors.ValidationError, scheduler.Description, self.dsc_path)

    def testNoSuchSection(self):
        self.utils.injectPayload(payload=Generator.getDefaultPayload())
        dsc = scheduler.Description(self.dsc_path)
        try:
            dsc["FOOBAR"]
        except KeyError:
            pass
        else:
            self.fail("Getting a non-existant section does not raise an exception")

    def testIteration(self):
        self.utils.injectPayload(payload=Generator.getDefaultPayload())
        dsc = scheduler.Description(self.dsc_path)

        # these are the option in the default payload
        expected = ['levels', 'groups', 'description', 'deps', 'prom2', 'file_name', 'prom1', 'name']
        options = [ opt for opt in dsc["HEADER"] ]
        self.assertEqual(options.sort(),expected.sort())

    def testGroups(self):
        # FIXME - add some code here
        pass

class Test_ST_TestLoader(unittest.TestCase, Generator):
    def setUp(self):
        self.directory = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.directory)
        #print self.directory

    def testCorrectLoad(self):
        dsc_fname = 'testCorrectLoad.dsc'
        tst_fname = Generator.defaults["file_name"]
        expected = [ ( os.path.join(self.directory, tst_fname), os.path.join(self.directory, dsc_fname) ) ]

        self.insertNamedFile(name=dsc_fname, payload=Generator.getDefaultPayload())
        self.insertNamedFile(name=tst_fname, executable=True)

        finder = scheduler.ST_TestLoader()
        suite = finder.loadFromPath(self.directory, self.directory)

        # test we got the one test we wanted to have
        self.assertEqual(finder.findDescriptions(self.directory), [ os.path.join(self.directory, dsc_fname) ] )
        # see if one test got loaded into the suite
        self.assertEqual(suite.countTestCases(), 1)

    def testNoDescription(self):
        finder = scheduler.ST_TestLoader()

        # test that the list is empty
        self.assertEqual(finder.findDescriptions(self.directory), list())
        # test that the suite returned is empty
        suite = finder.loadFromPath(self.directory, self.directory)
        self.assertEqual(suite.countTestCases(), 0)

    def testNoSuchPath(self):
        finder = scheduler.ST_TestLoader()
        # test that the list is empty
        self.assertRaises(IOError, finder.findDescriptions, "foo/bar")
        # test that the suite returned is empty
        self.assertRaises(IOError, finder.loadFromPath, "foo/bar", "foo/bar")

    def testFilterNotCallable(self):
        self.assertRaises(TypeError, scheduler.ST_TestLoader, filter="foo")

    def my_filter(self, desc):
        return desc.fileName.endswith('return_me.dsc')

    def testFilterBasic(self):
        good_dsc_fname  = 'return_me.dsc'
        bad_dsc_fname   = 'foobar.dsc'
        good_tst_fname  = Generator.defaults["file_name"]

        self.insertNamedFile(name=good_dsc_fname, payload=Generator.getDefaultPayload())
        self.insertNamedFile(name=bad_dsc_fname, payload=Generator.getDefaultPayload())
        self.insertNamedFile(name=good_tst_fname, executable=True)

        finder = scheduler.ST_TestLoader(filter_func=self.my_filter)
        suite = finder.loadFromPath(self.directory, self.directory)
        self.assertEqual(suite.countTestCases(), 1)

    def testFilterIncludeTags(self):
        select_list = []

        def wrapper(dsc):
            return scheduler.ST_TestLoader.filterIncludeTags(dsc, select_list)

        groups1 = "group1 group2"
        groups2 = "group2 group3"
        good_tst_fname  = Generator.defaults["file_name"]

        payload1 = Generator.customizeDescription(groups=groups1)
        payload2 = Generator.customizeDescription(groups=groups2)

        self.insertNamedFile(name="test1.dsc", payload=payload1)
        self.insertNamedFile(name="test2.dsc", payload=payload2)
        self.insertNamedFile(name=good_tst_fname, executable=True)

        finder = scheduler.ST_TestLoader(filter_func=wrapper)

        select_list = [ "group1" ]
        suite = finder.loadFromPath(self.directory, self.directory)
        self.assertEqual(suite.countTestCases(), 1) # only test1 contains tag group1

        select_list = [ "group2" ]
        suite = finder.loadFromPath(self.directory, self.directory)
        self.assertEqual(suite.countTestCases(), 2) # both tests contain tag group2

    def testFilterNameContains(self):
        name_contains = ""
        def wrapper(dsc):
            return scheduler.ST_TestLoader.filterNameContains(dsc, name_contains)

        name1 = "testa1"
        name2 = "testb2"
        good_tst_fname  = Generator.defaults["file_name"]

        payload1 = Generator.customizeDescription(name=name1)
        payload2 = Generator.customizeDescription(name=name2)

        self.insertNamedFile(name="test1.dsc", payload=payload1)
        self.insertNamedFile(name="test2.dsc", payload=payload2)
        self.insertNamedFile(name=good_tst_fname, executable=True)

        finder = scheduler.ST_TestLoader(filter_func=wrapper)

        name_contains = "test"
        suite = finder.loadFromPath(self.directory, self.directory)
        self.assertEqual(suite.countTestCases(), 2) # both tests contain "test" in their name

        name_contains = "testa"
        suite = finder.loadFromPath(self.directory, self.directory)
        self.assertEqual(suite.countTestCases(), 1) # only one test contains "testa"

        name_contains = "xxx"
        suite = finder.loadFromPath(self.directory, self.directory)
        self.assertEqual(suite.countTestCases(), 0) # no test contains "xxx"

    def testFilterNameEquals(self):
        name_equals = ""
        def wrapper(dsc):
            return scheduler.ST_TestLoader.filterNameEquals(dsc, name_equals)

        name1 = "testa1"
        name2 = "testb2"
        good_tst_fname  = Generator.defaults["file_name"]

        payload1 = Generator.customizeDescription(name=name1)
        payload2 = Generator.customizeDescription(name=name2)

        self.insertNamedFile(name="test1.dsc", payload=payload1)
        self.insertNamedFile(name="test2.dsc", payload=payload2)
        self.insertNamedFile(name=good_tst_fname, executable=True)

        finder = scheduler.ST_TestLoader(filter_func=wrapper)

        name_equals = name1
        suite = finder.loadFromPath(self.directory, self.directory)
        self.assertEqual(suite.countTestCases(), 1) # return the test named "testa1"

    def testFilterDescriptionContains(self):
        desc_contains = ""
        def wrapper(dsc):
            return scheduler.ST_TestLoader.filterDescriptionContains(dsc, desc_contains)

        desc1 = "bucket fried chicken"
        desc2 = "comes in real buckets"
        good_tst_fname  = Generator.defaults["file_name"]

        payload1 = Generator.customizeDescription(description=desc1)
        payload2 = Generator.customizeDescription(description=desc2)

        self.insertNamedFile(name="test1.dsc", payload=payload1)
        self.insertNamedFile(name="test2.dsc", payload=payload2)
        self.insertNamedFile(name=good_tst_fname, executable=True)

        finder = scheduler.ST_TestLoader(filter_func=wrapper)

        desc_contains = "bucket"
        suite = finder.loadFromPath(self.directory, self.directory)
        self.assertEqual(suite.countTestCases(), 2) # both tests contain "bucket" in their desc

        desc_contains = "chicken"
        suite = finder.loadFromPath(self.directory, self.directory)
        self.assertEqual(suite.countTestCases(), 1) # only one test contains "chicken"

        desc_contains = "xxx"
        suite = finder.loadFromPath(self.directory, self.directory)
        self.assertEqual(suite.countTestCases(), 0) # no test contains "xxx"

    def testFindDescriptions(self):
        testnames = [ "test1.dsc", "test2.dsc" ]
        self.insertNamedFile(name="test1.dsc", payload=Generator.customizeDescription())
        self.insertNamedFile(name="test2.dsc", payload=Generator.customizeDescription())
        finder = scheduler.ST_TestLoader()
        self.assertEqual(len([ path for path in finder.findDescriptions(self.directory) if os.path.basename(path) not in testnames ]), 0)

class Test_ST_TestSuite(unittest.TestCase, Generator):
    def setUp(self):
        self.directory = tempfile.mkdtemp()
        self.gen = Generator(None, self.directory)

    def tearDown(self):
        shutil.rmtree(self.directory)

    def testAddTestCase(self):
        dsc_path = self.gen.createSimpleTestCase(self.directory)
        case = scheduler.ST_TestCase(scheduler.Description(dsc_path))

        suite = scheduler.ST_TestSuite()
        suite.addTest(case)
        
        self.assertEqual(suite.countTestCases(), 1)

    def testAddMoreTestCases(self):
        dsc_path = self.gen.createSimpleTestCase(self.directory)
        case_list = [ scheduler.ST_TestCase(scheduler.Description(dsc_path)) for i in xrange(0,4) ]

        suite = scheduler.ST_TestSuite()
        suite.addTests(case_list)
        
        self.assertEqual(suite.countTestCases(), 4)

    def testAddOperator(self):
        dsc_path = self.gen.createSimpleTestCase(self.directory)
        case = scheduler.ST_TestCase(scheduler.Description(dsc_path))

        suite = scheduler.ST_TestSuite()
        suite.addTest(case)

        suite2 = scheduler.ST_TestSuite()

        suite2 += suite
        self.assertEqual(suite.countTestCases(), 1)

    def testRunSuite(self):
        dsc_path = self.gen.createSimpleTestCase(self.directory)
        case = scheduler.ST_TestCase(scheduler.Description(dsc_path))

        suite = scheduler.ST_TestSuite()
        suite.addTest(case)
        case.assert_executable(os.path.split(dsc_path)[0])
        suite()

    def testSuiteFiltering(self):
        dsc_path = self.gen.createSimpleTestCase(self.directory)
        case = scheduler.ST_TestCase(scheduler.Description(dsc_path))

        # the suite now contains one test called 'test'
        suite = scheduler.ST_TestSuite()
        suite.addTest(case)
        self.assertEquals(suite.countTestCases(), 1)

        # no test 'foo' -> an empty suite
        filtered = suite.filter(scheduler.ST_TestLoader.filterNameEquals, name = 'foo')
        self.assertEquals(filtered.countTestCases(), 0)

        # should return the same suite
        filtered = suite.filter(scheduler.ST_TestLoader.filterNameEquals, name = 'test')
        self.assertEquals(filtered.countTestCases(), 1)

class Test_ST_TestCase(unittest.TestCase, Generator):
    def setUp(self):
        self.directory = tempfile.mkdtemp()
        self.gen = Generator(None, self.directory)

    def tearDown(self):
        shutil.rmtree(self.directory)

    def testLoadDescription(self):
        dsc_path = self.gen.createSimpleTestCase(self.directory)
        case = scheduler.ST_TestCase(scheduler.Description(dsc_path))

        repr =  "<%s (%s) -> %s>" % (dsc_path, case.dsc["HEADER"]["NAME"], case.dsc["HEADER"]["FILE_NAME"])
        self.assertEqual(repr, str(case.dsc))

    def testProvideDescription(self):
        dsc_path = self.gen.createSimpleTestCase(self.directory)
        dsc = scheduler.Description(dsc_path)
        case = scheduler.ST_TestCase(scheduler.Description(dsc_path))

        repr =  "<%s (%s) -> %s>" % (dsc_path, case.dsc["HEADER"]["NAME"], case.dsc["HEADER"]["FILE_NAME"])
        self.assertEqual(repr, str(case.dsc))

    def testScriptBadPath(self):
        path = self.insertNamedFile(name="noScript.dsc", payload=Generator.customizeDescription(name="no_such_script.sh"))
        case = scheduler.ST_TestCase(scheduler.Description(path))
        self.assertRaises(errors.PeerError, case.assert_executable, os.path.split(path)[0])

    def testScriptPassLevel(self):
        " test if $LEVEL is passed to the test "
        default_env_var_test = """
        #!/bin/bash
        if [ x"$LEVEL" == "x" ]; then
            exit 1 
        fi
        exit 0"""

        test_name = "env_test.sh"
        dsc_path = self.insertNamedFile(name="env_test.dsc", payload=Generator.customizeDescription(file_name=test_name))
        self.insertNamedFile(name=test_name, executable=True, payload=default_env_var_test)

        case = scheduler.ST_TestCase(scheduler.Description(dsc_path))
        case.assert_executable(os.path.split(dsc_path)[0])
        result = case()
        self.assertEqual(len(result.failures), 0)
        self.assertEqual(result.return_code, 0)

    def testScriptPassEnvironment(self):
        default_env_var_test = """
        #!/bin/bash
        if [ "$PROM1" == "value1" ]; then
            exit 0
        fi
        exit 1"""

        test_name = "env_test.sh"
        dsc_path = self.insertNamedFile(name="env_test.dsc", payload=Generator.customizeDescription(file_name=test_name))
        self.insertNamedFile(name=test_name, executable=True, payload=default_env_var_test)

        case = scheduler.ST_TestCase(scheduler.Description(dsc_path))
        case.assert_executable(os.path.split(dsc_path)[0])
        result = case()
        self.assertEqual(len(result.failures), 0)
        self.assertEqual(result.return_code, 0)

    def testErrorInScript(self):
        """ Test that the runner is able to detect errors in the script worker """
        bad_test = """#!/bin/bash
        if [x"$TERM" == "x" ] then

        exit 1
        """
        test_name = "bad_test.sh"
        dsc_path = self.insertNamedFile(name="bad_test.dsc", payload=Generator.customizeDescription(file_name=test_name))
        self.insertNamedFile(name=test_name, executable=True, payload=bad_test)

        case = scheduler.ST_TestCase(scheduler.Description(dsc_path))
        case.assert_executable(os.path.split(dsc_path)[0])
        case()
        self.assertNotEqual(len(case.result.stderr), 0)

    def testInvalidLevel(self):
        dsc_path = self.gen.createVariableTestCase(self.directory, "test.dsc", "NAME", Generator.defaults["name"])
        case = scheduler.ST_TestCase(scheduler.Description(dsc_path))
        self.assertRaises(ValueError, case.runTestLevel, 100)

    def testEraseEnvironment(self):
        """ Test that the environment for a test is set from the scratch """
        os.putenv("EDITOR", "ed")
        default_env_var_test = """
        #!/bin/bash
        if [ x"$EDITOR" == "x" ]; then
            exit 0
        fi
        env

        exit 1
        """

        test_name = "env_test.sh"
        dsc_path = self.insertNamedFile(name="env_test.dsc", payload=Generator.customizeDescription(file_name=test_name))
        self.insertNamedFile(name=test_name, executable=True, payload=default_env_var_test)

        case = scheduler.ST_TestCase(scheduler.Description(dsc_path))
        case.assert_executable(os.path.split(dsc_path)[0])
        result = case()
        print result.unmatched
        self.assertEqual(len(result.failures), 0)
        self.assertEqual(result.return_code, 0)

    def testCheckForDependencies(self):
        """ Test if testLoader is able to detect missing dependencies """
        # valid deps should not raise exception
        path = self.insertNamedFile(name="valid_deps.dsc", payload=Generator.customizeDescription(deps="bash awk"))
        test = self.insertNamedFile(name=Generator.defaults["file_name"], executable=True)
        tcase = scheduler.ST_TestCase(scheduler.Description(path))
        tcase.setUp()

        # invalid deps - should raise errors.MissingDependencyError
        path = self.insertNamedFile(name="invalid_deps.dsc", payload=Generator.customizeDescription(deps="foo bar"))
        tcase = scheduler.ST_TestCase(scheduler.Description(path))
        self.assertRaises(errors.MissingDependencyError, tcase.setUp)

        # valid package dependency - should not raise an exception
        path = self.insertNamedFile(name="valid_pkg_deps.dsc", payload=Generator.customizeDescription(deps="pkg(bash)"))
        tcase = scheduler.ST_TestCase(scheduler.Description(path))
        tcase.setUp()

        # missing package dependency - should raise errors.MissingDependencyError
        path = self.insertNamedFile(name="missing_pkg_deps.dsc", payload=Generator.customizeDescription(deps="pkg(froblap)"))
        tcase = scheduler.ST_TestCase(scheduler.Description(path))
        self.assertRaises(errors.MissingDependencyError, tcase.setUp)

        # valid absolute path dependency - should not raise an exception
        path = self.insertNamedFile(name="valid_absolute_path_dep.dsc", payload=Generator.customizeDescription(deps="/bin/bash"))
        tcase = scheduler.ST_TestCase(scheduler.Description(path))
        tcase.setUp()

        # invalid absolute path dependency - should raise errors.MissingDependencyError
        path = self.insertNamedFile(name="invalid_absolute_path_dep.dsc", payload=Generator.customizeDescription(deps="/foo/bar"))
        tcase = scheduler.ST_TestCase(scheduler.Description(path))
        self.assertRaises(errors.MissingDependencyError, tcase.setUp)

        # mixed dependencies which are OK
        path = self.insertNamedFile(name="real_life_deps.dsc", payload=Generator.customizeDescription(deps="/bin/bash pkg(bash) mount pkg(coreutils) /bin/cp"))
        tcase = scheduler.ST_TestCase(scheduler.Description(path))
        tcase.setUp()

    def testNoDependenciesListedInDsc(self):
        path = self.insertNamedFile(name="valid_deps.dsc", payload=Generator.customizeDescription(deps=""))
        test = self.insertNamedFile(name=Generator.defaults["file_name"], executable=True)
        tcase = scheduler.ST_TestCase(scheduler.Description(path))
        tcase.setUp()

    def testGrabReturnCode(self):
        return_value = """
        #!/bin/bash
        exit 99
        """

        test_name = "return_value.sh"
        dsc_path = self.insertNamedFile(name="return_value.dsc", payload=Generator.customizeDescription(file_name=test_name))
        self.insertNamedFile(name=test_name, executable=True, payload=return_value)

        case = scheduler.ST_TestCase(scheduler.Description(dsc_path))
        case.assert_executable(os.path.split(dsc_path)[0])
        result = case()
        self.assertEquals(result.return_code, 99)

    def testCreateDataDirectory(self):
        tdata_base = '/tmp'
        path = self.insertNamedFile(name="data_dir.dsc", payload=Generator.customizeDescription(data_dir=1))
        test = self.insertNamedFile(name=Generator.defaults["file_name"], executable=True)
        tcase = scheduler.ST_TestCase(scheduler.Description(path))
        tcase.environ.update({'TDATA_DIR_BASE' : tdata_base })
        tcase.setUp()
        self.assert_(os.path.exists(os.path.join(tdata_base, Generator.defaults["name"])))
        os.rmdir(os.path.join(tdata_base, Generator.defaults["name"]))

    def testRemoveDataDirectory(self):
        tdata_base = '/tmp'
        path = self.insertNamedFile(name="data_dir.dsc", payload=Generator.customizeDescription(data_dir=1))
        test = self.insertNamedFile(name=Generator.defaults["file_name"], executable=True)
        tcase = scheduler.ST_TestCase(scheduler.Description(path))
        tcase.environ.update({'TDATA_DIR_BASE' : tdata_base })
        tcase.setUp()
        tcase.remove_data_dir()
        self.assert_(not os.path.exists(os.path.join(tdata_base, Generator.defaults["name"])))

class TestResultDiff(unittest.TestCase):
    def setUp(self):
        self.before = scheduler.ST_TestResult("test-before")
        self.after = scheduler.ST_TestResult('test-after')
        self.differ = scheduler.ST_TestResultDiff(self.before, self.after)

    def testItemAdded(self):
        self.before.addWarning(1, 'This is a warning')

        self.after.addWarning(1, 'This is a warning')
        self.after.addError(2, 'This is an error')

        self.assertEqual(self.differ.diff(), False)
        self.assertEqual(len(self.differ.removed), 0)
        self.assertEqual(len(self.differ.added), 1)

    def testItemRemoved(self):
        self.before.addWarning(1, 'This is a warning')
        self.before.addError(2, 'This is an error')

        self.after.addWarning(1, 'This is a warning')

        self.assertEqual(self.differ.diff(), False)
        self.assertEqual(len(self.differ.removed), 1)
        self.assertEqual(len(self.differ.added), 0)

    def testSameResults(self):
        self.before.addWarning(1, 'This is a warning')
        self.after.addWarning(1, 'This is a warning')

        self.assertEqual(self.differ.diff(), True)
        self.assertEqual(len(self.differ.removed), 0)
        self.assertEqual(len(self.differ.added), 0)

    def testAddedAndRemoved(self):
        self.before.addWarning(1, 'This is a warning')
        self.before.addError(2, 'This is an error')
        self.after.addWarning(1, 'This is a warning')
        self.after.addError(3, 'Some other error')

        self.assertEqual(self.differ.diff(), False)
        self.assertEqual(len(self.differ.added), 1)
        self.assertEqual(len(self.differ.removed), 1)

    def testSameTextDifferentId(self):
        self.before.addWarning(1, 'This is a warning')
        self.after.addWarning(2, 'This is a warning')

        self.assertEqual(self.differ.diff(), False)
        self.assertEqual(len(self.differ.added), 1)
        self.assertEqual(len(self.differ.removed), 1)

    def testSameIdDifferentText(self):
        self.before.addWarning(1, 'This is a warning')
        self.after.addWarning(1, 'This is another warning')

        self.assertEqual(self.differ.diff(), False)
        self.assertEqual(len(self.differ.added), 1)
        self.assertEqual(len(self.differ.removed), 1)

class TestXmlOutput(unittest.TestCase):
    def setUp(self):
        self.result = scheduler.ST_TestResult('test-xml-output')
        self.result.addWarning(1, 'This is a warning')
        self.result.addError(2, 'This is an error')
        self.result.addHint(2, 'This is a hint')
        self.result.addInfo(2, 'This is an info')

        self.outfile, self.outfile_path = tempfile.mkstemp()
        try:
            self.xml_out = xmloutput.XmlOutput(self.outfile_path)
        except ExpatError, e:
            print e.code

    def tearDown(self):
        #print self.outfile_path
        os.remove(self.outfile_path)

    def testInitEmptyFile(self):
        expected = ['<?xml version="1.0" encoding="utf-8"?>\n', '<results/>\n']
        self.assertEqual(open(self.outfile_path).readlines(), expected)

    def testNoSuchFile(self):
        self.assertRaises(IOError, xmloutput.XmlOutput, '/no/such/file')

    def testCorruptFile(self):
        content = ['<?xml version="1.0" encoding="utf-8"?>\n', '<results>\n', '\t<test>\n', '\t\t<name>\n', 
                    '\t\t\tsome_test\n', '\t\t</name>\n', '\t\t<result>\n', '\t\t\t2\n',
                    '\t\t</result>\n', '\t\t<messages>\n', '\t\t\t<msg id="1" type="2">\n',
                    '\t\t\t\tThis is a warning\n', '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="1">\n',
                    '\t\t\t\tThis is an error\n', '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="3">\n', 
                    '\t\t\t\tThis is a hint\n', '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="4">\n', 
                    '\t\t\t\tThis is an info\n', '\t\t\t</msg>\n', '\t\t</messages>\n'      ]
        open(self.outfile_path, 'w').writelines(content)
        self.assertRaises(ExpatError, xmloutput.XmlOutput, self.outfile_path)

    def testAppendResult(self):
        expected = ['<?xml version="1.0" encoding="utf-8"?>\n', '<results>\n', '\t<test>\n', '\t\t<name>\n', 
                    '\t\t\ttest_1\n', '\t\t</name>\n', '\t\t<return_code>\n', '\t\t\t-1\n', 
                    '\t\t</return_code>\n', '\t\t<result>\n', '\t\t\t2\n', '\t\t</result>\n', 
                    '\t\t<messages>\n', '\t\t\t<msg id="1" type="2">\n', '\t\t\t\tThis is a warning\n', 
                    '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="1">\n', '\t\t\t\tThis is an error\n', 
                    '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="3">\n', '\t\t\t\tThis is a hint\n', 
                    '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="4">\n', '\t\t\t\tThis is an info\n', 
                    '\t\t\t</msg>\n', '\t\t</messages>\n', '\t</test>\n', '\t<test>\n', '\t\t<name>\n', 
                    '\t\t\ttest_2\n', '\t\t</name>\n', '\t\t<return_code>\n', '\t\t\t-1\n', '\t\t</return_code>\n', 
                    '\t\t<result>\n', '\t\t\t2\n', '\t\t</result>\n', '\t\t<messages>\n', '\t\t\t<msg id="1" type="2">\n', 
                    '\t\t\t\tThis is some other warning\n', '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="1">\n', 
                    '\t\t\t\tThis is some other error\n', '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="1">\n', 
                    '\t\t\t\tYet another error\n', '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="3">\n', 
                    '\t\t\t\tThis is some other hint\n', '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="4">\n', 
                    '\t\t\t\tThis is some other info\n', '\t\t\t</msg>\n', '\t\t</messages>\n', '\t</test>\n', '</results>\n']

        result2 = scheduler.ST_TestResult('test-append-result')
        result2.addWarning(1, 'This is some other warning')
        result2.addError(2, 'This is some other error')
        result2.addError(2, 'Yet another error')
        result2.addHint(2, 'This is some other hint')
        result2.addInfo(2, 'This is some other info')

        self.xml_out.write_result('test_1', self.result)
        self.xml_out.write_result('test_2', result2)

        self.assertEqual(open(self.outfile_path).readlines(), expected)

    def testOutputResult(self):
        expected = ['<?xml version="1.0" encoding="utf-8"?>\n', '<results>\n', '\t<test>\n', '\t\t<name>\n', 
                    '\t\t\tsome_test\n', '\t\t</name>\n', '\t\t<return_code>\n', '\t\t\t-1\n', 
                    '\t\t</return_code>\n', '\t\t<result>\n', '\t\t\t2\n', '\t\t</result>\n', 
                    '\t\t<messages>\n', '\t\t\t<msg id="1" type="2">\n', '\t\t\t\tThis is a warning\n', 
                    '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="1">\n', '\t\t\t\tThis is an error\n', 
                    '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="3">\n', '\t\t\t\tThis is a hint\n', 
                    '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="4">\n', '\t\t\t\tThis is an info\n', 
                    '\t\t\t</msg>\n', '\t\t</messages>\n', '\t</test>\n', '</results>\n']

        self.xml_out.write_result('some_test', self.result)
        self.assertEqual(open(self.outfile_path).readlines(), expected)

    def testSubstituteResult(self):
        expected = ['<?xml version="1.0" encoding="utf-8"?>\n', '<results>\n', '\t<test>\n', '\t\t<name>\n', 
                    '\t\t\tsome_test\n', '\t\t</name>\n', '\t\t<return_code>\n', '\t\t\t-1\n', 
                    '\t\t</return_code>\n', '\t\t<result>\n', '\t\t\t2\n', '\t\t</result>\n', 
                    '\t\t<messages>\n', '\t\t\t<msg id="1" type="2">\n', '\t\t\t\tThis is some other warning\n', 
                    '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="1">\n', '\t\t\t\tThis is some other error\n', 
                    '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="1">\n', '\t\t\t\tYet another error\n', 
                    '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="3">\n', '\t\t\t\tThis is some other hint\n', 
                    '\t\t\t</msg>\n', '\t\t\t<msg id="2" type="4">\n', '\t\t\t\tThis is some other info\n', 
                    '\t\t\t</msg>\n', '\t\t</messages>\n', '\t</test>\n', '</results>\n']

        result2 = scheduler.ST_TestResult('test-substitute-result')
        result2.addWarning(1, 'This is some other warning')
        result2.addError(2, 'This is some other error')
        result2.addError(2, 'Yet another error')
        result2.addHint(2, 'This is some other hint')
        result2.addInfo(2, 'This is some other info')

        self.xml_out.write_result('some_test', self.result)
        self.xml_out.write_result('some_test', result2)

        self.assertEqual(open(self.outfile_path).readlines(), expected)

    def testMultipleRuns(self):
        names = [ 'one', 'two' ]
        for name in names:
            self.xml_out.write_result(name, self.result)
            self.xml_out = xmloutput.XmlOutput(self.outfile_path)

    def testXmlToResult(self):
        xml_result = self.xml_out._XmlOutput__result2xml('foo', self.result)
        real_result = self.xml_out._XmlOutput__xml2result(xml_result)

        self.assertEqual(len(real_result.output_items), len(self.result.output_items))
        self.assertEqual(real_result.return_code, self.result.return_code)
        self.assertEqual(real_result.result, self.result.result)

    def testNonAsciiInput(self):
        result = scheduler.ST_TestResult('test-non-ascii-input')
        result.addError(2, "ěščřžýá")

        self.xml_out.write_result('some_test', result)
        self.assert_(len( [ line for line in open(self.outfile_path).readlines() if "ěščřžýá" in line ]))

    def testCP1250Input(self):
        result = scheduler.ST_TestResult('test-cp1250-input')
        result.addError(2, unicode("this is windows input - ěščřžýá", 'UTF-8').encode('WINDOWS-1250'))

        self.xml_out.write_result('some_test', result)
        self.assert_(len( [ line for line in open(self.outfile_path).readlines() if "this is windows input - " in line ]))

    
class TestLevelOverriding(unittest.TestCase):
    def setUp(self):
        self.rcfile, self.rcfile_path = tempfile.mkstemp()
        self.rcfile_handle = os.fdopen(self.rcfile, "w")
        self.test_repo = tempfile.mkdtemp()

        gen = Generator(None, self.test_repo)
        gen.insertNamedFile(name="test12.dsc", payload = Generator.customizeDescription(levels = "1 2", name = "test12"))
        gen.insertNamedFile(name="test34.dsc", payload = Generator.customizeDescription(levels = "3 4", name = "test34"))
        gen.insertNamedFile(name="test45.dsc", payload = Generator.customizeDescription(levels = "4 5", name = "test45"))
        gen.insertNamedFile(name="script.sh", executable=True)

    def tearDown(self):
        #print self.rcfile_path
        #print self.test_repo
        os.remove(self.rcfile_path)
        shutil.rmtree(self.test_repo)

    @staticmethod
    def relative_filter(test, **kwargs):
        level = kwargs['level']

        if level in test.valid_levels:
            return True

        return False

    def cnt_loaded_tests(self, load_level):
        loader = scheduler.ST_TestLoader(level_config=self.rcfile_path)
        suite = loader.loadFromPath(self.test_repo, self.test_repo)
        suite.filterInPlace(TestLevelOverriding.relative_filter,
                            level = load_level)

        return len(suite)

    def write_rcfile(self, hash):
        for test, levels in hash.items():
            self.rcfile_handle.writelines("%s\t%s\n" % (test, levels))
        self.rcfile_handle.flush()

    def testAddLevel(self):
        self.assertEqual(self.cnt_loaded_tests(2), 1)
        self.write_rcfile( { "test34" : "2 3" } )
        self.assertEqual(self.cnt_loaded_tests(2), 2)

    def testAddNonDefaultLevel(self):
        # add level 1 into test34 - not defined with [level1] in the dsc file
        self.assertEqual(self.cnt_loaded_tests(1), 1)
        self.write_rcfile( { "test34" : "1 3 4" } )
        self.assertEqual(self.cnt_loaded_tests(1), 2)

    def testRemoveLevel(self):
        self.assertEqual(self.cnt_loaded_tests(4), 2)
        self.write_rcfile( { "test34" : "3" } )
        self.assertEqual(self.cnt_loaded_tests(4), 1)

    def testNonIntegerLevel(self):
        self.write_rcfile( { "test34" : "x" } )
        self.assertRaises(errors.SectoolrcError, scheduler.ST_TestLoader, level_config=self.rcfile_path)

    def testTooManyLevels(self):
        self.write_rcfile( { "test34" : "1 2 3 4 5 1" } )
        self.assertRaises(errors.SectoolrcError, scheduler.ST_TestLoader, level_config=self.rcfile_path)

    def testRemoveAllLevels(self):
        self.assertEqual(self.cnt_loaded_tests(4), 2)
        self.write_rcfile( { "test34" : " " } )
        self.assertEqual(self.cnt_loaded_tests(4), 1)


class TestIncludeExclude(unittest.TestCase):
    # FIXME - add some code here
    pass

# -------------- run the test suite -------------- #
if __name__ == "__main__":
    unittest.main()




