#!/usr/bin/python -t
#coding=utf-8

import unittest
import subprocess
import os
import sys
import exceptions
import tempfile
import shutil

from scheduler import scheduler
import sectool
from mailoutput import MailOutput

SECTOOL_BIN="/usr/sbin/sectool"

###############################################################################
#                                                                             #
#                          Auxilarry classes                                  #
#                                                                             #
###############################################################################

class GeneratorBase(object):
    def _customize_payload(self, skeleton, defaults, kwargs):
        all_params = defaults.copy()
        all_params.update(dict(kwargs))
        payload = skeleton % all_params

        # deindent
        payload = "\n".join( [ line.lstrip() for line in payload.split("\n") if line != "" ] )

        return payload

    def _write_payload(self, base_dir, file_name, content):
        path = os.path.join(base_dir, file_name)
        try:
            fhandle = open(path, 'w')
            assert fhandle != None
            fhandle.write(content)
            fhandle.flush()
        except IOError, e:
            print >>sys.stderr, "Could not write content of file %s, check path and/or permissions" % path
            print >>sys.stderr, "The error was: %s" % str(e)
            return None

        return fhandle

class ConfigGenerator(GeneratorBase):
    config_skeleton = """
    [ACTION]
    LEVEL=%(level)s

    [PATHS]
    DSC_DIR=%(dsc_dir)s
    TESTS_DIRS=%(tests_dir)s
    TDATA_DIR_BASE=%(tdata_dir_base)s
    TEMP_DIR=%(temp_dir)s

    [RESULTS]
    RESULT_FILE=%(result_file)s

    [ENVIRONMENT]
    PATH=%(path)s

    [MAIL]
    SEND_BODY=%(send_body)s
    SEND_ATTACHMENT=%(send_attachment)s
    TARGET=%(target)s
"""

    config_defaults = {
            'level'     : 3,
            'dsc_dir'   : '/etc/sectool/tests',
            'tests_dir' : '/usr/share/sectool/tests',
            'tdata_dir_base' : '/var/lib/sectool',
            'temp_dir'  : '/tmp',
            'result_file' : 'results.xml',
            'path'      : '/sbin:/bin:/usr/sbin:/usr/bin',
            'send_body' : 'diff',
            'send_attachment' : 'full',
            'target'    : 'local'
    }

    def write_config(self, path='/etc/sectool/sectool.conf', **kwargs):
        content = self._customize_payload(self.config_skeleton, self.config_defaults, kwargs)
        fpath, fname = os.path.split(path)
        file = self._write_payload(fpath, fname, content)

        return file.name

class TestGenerator(GeneratorBase):
    dsc_skeleton = """
    [HEADER]
    NAME=\"%(name)s\"
    FILE_NAME=\"%(file_name)s\"
    DESCRIPTION=\"%(description)s\"
    LEVELS=\"%(levels)s\"
    GROUPS=\"%(groups)s\"
    DEPS=\"%(deps)s\"
    REQUIRES=\"%(requires)s\"
    NEED_DATA_DIR=%(data_dir)s

    [DEFAULT]
    PROM1="value1"
    PROM2="value2"

    [LEVEL2]
    PROM1="value1_level2"
    """

    dsc_defaults = { "name" : "test",
                 "file_name" : "script.sh",
                 "description" : "A test description",
                 "levels" : "1 2 3 4 5",
                 "groups" : "general",
                 "deps" : "bash find",
                 "requires" : "bash",
                 "data_dir" : 0
               }

    def __init__(self, dsc_dir, tests_dir):
        self.dsc_dir = dsc_dir
        self.tests_dir = tests_dir

    def create_dsc(self, dsc_file_name, **kwargs):
        content = self._customize_payload(self.dsc_skeleton, self.dsc_defaults, kwargs)
        self._write_payload(self.dsc_dir, dsc_file_name, content)

    def remove_dsc(self, file_name):
        path = os.path.join(self.dsc_dir, file_name)
        try:
            os.remove(path)
        except:
            print "Could not remove DSC file %s please check path and permissions" % path

    def create_simple_test(self, file_name='script.sh', exit_value=0):
        script_payload = """
        #!/bin/bash -x

        ls -ld /tmp
        exit %d
        """

        test = self._write_payload(self.tests_dir, file_name, script_payload % exit_value)
        os.chmod(test.name,0755)

    def create_named_test(self, test_name):
        self.create_dsc(test_name + '.dsc', name = test_name, file_name = test_name + '.sh')
        self.create_simple_test(file_name = "%s.sh" % test_name)

class SectoolRunner(object):
    def __init__(self, config_file='/etc/sectool/sectool.conf'):
        self.config_file = config_file

        self.warn  = []
        self.error = []
        self.hint  = []
        self.info  = []

        self.stderr = []
        self.other  = []
        self.header = ""
        self.footer = ""

    def filter(self, output):
        hash = { 'Error:'  : self.error,
                 'Warning:': self.warn,
                 'Hint:'   : self.hint,
                 'Info:'   : self.info }

        test_name = ""

        for l in output:
            line = l.strip()
            splitted = line.split()
            if len(splitted) == 0: continue

            # this is fragile. Regular expressions might be better
            if '->' in line:
                test_name = splitted[0]
                self.header = line

            if "%s: " % (test_name) in line:
                self.footer = line

            if "stderr from " in line:
                self.stderr.append(line)

            tag = splitted[0]
            try:
                where = hash[tag]
            except KeyError:
                tag = ""
                where = self.other
            where.append(line[len(tag):].lstrip())

    def run_sectool(self, args):
        p = subprocess.Popen("%s %s" % (SECTOOL_BIN, args), 
                             shell=True,
                             bufsize=4096,
                             close_fds=True,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)

        return (p.wait(), p.stdout.readlines() + p.stderr.readlines())

class SectoolTestCase(unittest.TestCase):
    def setUp(self):
        unittest.TestCase.setUp(self)
        self.runner = SectoolRunner()

    def runTestLevel(self):
        return self.runner.run("--run %s --level %d" % (self.test, self.level))

    def runSectool(self, args, expect_failure=False):
        rc, output = self.runner.run_sectool(args)
        if expect_failure == False:
            self.assert_(rc == 0)
        self.runner.filter(output)

    def grepMatch(self, regex, files):
        p = subprocess.Popen("/bin/grep %s %s" % (regex, files), 
                             shell=True,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)

        return (p.wait(), p.stdout.readlines() + p.stderr.readlines())

    def _has_msg(self, list, msg):
        for list_msg in list:
            if msg in list_msg:
                return True

        return False

    def assertInfo(self, msg, contains=True):
        self.assert_(self._has_msg(self.runner.info, msg) == contains)

    def assertWarning(self, msg, contains=True):
        self.assert_(self._has_msg(self.runner.warning, msg) == contains)

    def assertError(self, msg, contains=True):
        self.assert_(self._has_msg(self.runner.error, msg) == contains)

    def assertHint(self, msg, contains=True):
        self.assert_(self._has_msg(self.runner.hint, msg) == contains)

    def assertStderr(self, msg, contains=True):
        self.assert_(self._has_msg(self.runner.stderr, msg) == contains)

    def assertStdout(self, msg, contains=True):
        self.assert_(self._has_msg(self.runner.other, msg) == contains)

###############################################################################
#                                                                             #
#                     Testing of sectool itself                               #
#                                                                             #
###############################################################################

class TestSanity(SectoolTestCase):
    " test totally basic stuff - --help, no arguments, ... "
    def testNoOptions(self):
        "test running sectool with no arguments: must print help message and tell user to select an action. Does not return 0."
        self.runSectool("", True)
        self.assertStdout('Usage: sectool command [arguments]')
        self.assertStdout('sectool: error: Must select at least one action')

    def testHelp(self):
        "test running sectool --help: must print help message"
        self.runSectool("--help")
        self.assertStdout('Usage: sectool command [arguments]')

class TestList(SectoolTestCase):
    def setUp(self):
        SectoolTestCase.setUp(self)
        self.workdir = tempfile.mkdtemp()
        conf = ConfigGenerator()
        self.config_file = conf.write_config(os.path.join(self.workdir, 'sectool.conf'),
                                             dsc_dir = self.workdir,
                                             tests_dir = self.workdir)
        self.tg = TestGenerator(self.workdir, self.workdir)

    def tearDown(self):
        SectoolTestCase.tearDown(self)
        shutil.rmtree(self.workdir)

    def testEmptyList(self):
        "sectool --list: test that --list is able to act upon empty repo"
        self.runSectool("-c %s --list" % self.config_file)
        # we should see just the header..4 lines
        self.assert_(len(self.runner.other) == 4)

    def testBasicList(self):
        "sectool --list: test that --list returns header and a few known tests"
        self.tg.create_named_test('blabla')
        self.tg.create_named_test('foobar')

        self.runSectool("-c %s --list" % self.config_file)
        self.assertStdout('blabla')
        self.assertStdout('foobar')

    def testListOneTest(self):
        "sectool --list <test>: should list only <test> even though there are two in the repo"
        self.tg.create_named_test('blabla')
        self.tg.create_named_test('foobar')

        self.runSectool("-c %s --list blabla" % self.config_file)
        # we should see blabla but not foobar
        self.assertStdout('blabla')
        self.assertStdout('foobar', False)

    def testListGlobbing(self):
        "sectool --list <test*>: should display only those tests that are in the repo"
        self.tg.create_named_test('test_one')
        self.tg.create_named_test('test_two')
        self.tg.create_named_test('foobar')

        self.runSectool("-c %s --list test\*" % self.config_file)
        # we should see both tests but not foobar
        self.assertStdout('test_one')
        self.assertStdout('test_two')
        self.assertStdout('foobar', False)

    def testListGlobbingAll(self):
        self.tg.create_named_test('test_one')
        self.tg.create_named_test('test_two')
        self.tg.create_named_test('foobar')

        self.runSectool("-c %s --list \*" % self.config_file)
        # we should see all tests
        self.assertStdout('test_one')
        self.assertStdout('test_two')
        self.assertStdout('foobar')

    def testListDisplayLevels(self):
        "sectool --list must correctly display levels"
        # define foobar in levels 1 and 2 and blabla in levels 4 and 5
        self.tg.create_dsc('foobar.dsc', levels = "1 2", name="foobar")
        self.tg.create_dsc('blabla.dsc', levels = "4 5", name="blabla")
        self.tg.create_simple_test()

        self.runSectool("-c %s --list" % self.config_file)

        # parse out lines for those two tests
        foobar_line = [ l for l in self.runner.other if l.startswith("foobar") ][0]
        blabla_line = [ l for l in self.runner.other if l.startswith("blabla") ][0]
        
        # check if they contain the correct level numbers
        self.assert_(foobar_line.strip().endswith("1 2"))
        self.assert_(blabla_line.endswith("4 5"))

    def testListRequires(self):
        "sectool --list should display * instead of level number in case requirements are not met"
        # create a test that requires package xyz, that likely won't be installed
        self.tg.create_dsc('foobar.dsc', name="foobar", requires="xyz")
        self.tg.create_simple_test()

        self.runSectool("-c %s --list" % self.config_file)

        # check that * instead of level numbers are displayed
        foobar_line = [ l for l in self.runner.other if l.startswith("foobar") ][0]
        self.assert_(foobar_line.strip().endswith("* * * * *"))

    def testNoSuchTest(self):
        " test listing a test that does not exist"
        # FIXME - should sectool say something like "No matching tests to display?"
        # FIXME - sectool also returns 0, that doesn't seem right
        self.runSectool("-c %s --list froblap" % self.config_file)
        self.assert_(len(self.runner.other) == 4)

class TestInfo(SectoolTestCase):
    def setUp(self):
        SectoolTestCase.setUp(self)
        self.workdir = tempfile.mkdtemp()
        #print self.workdir
        conf = ConfigGenerator()
        self.config_file = conf.write_config(os.path.join(self.workdir, 'sectool.conf'),
                                             dsc_dir = self.workdir,
                                             tests_dir = self.workdir)
        self.tg = TestGenerator(self.workdir, self.workdir)

    def tearDown(self):
        SectoolTestCase.tearDown(self)
        shutil.rmtree(self.workdir)

    def testNoTestsGiven(self):
        " test running sectool --info with no other arguments "
        # expect non-zero return code
        # FIXME - this currently goes to stdout, maybe it should go to stderr?
        self.runSectool("-c %s --info" % self.config_file, expect_failure=True)
        # expect just the error message
        self.assert_(len(self.runner.other) == 1)

    def testNoSuchTest(self):
        " test running sectool --info on non existent test"
        # FIXME - should sectool say something like "No matching tests to display?"
        # FIXME - sectool also returns 0, that doesn't seem right
        self.runSectool("-c %s --info nosuchtest" % self.config_file)
        self.assert_(len(self.runner.other) == 0)

    def testInfoOneTest(self):
        " test that --info displays info about one test "
        self.tg.create_named_test('foobar')
        self.runSectool("-c %s --info foobar" % self.config_file)

        expected = { 'Name' : 'foobar',
                     'Script name' : 'foobar.sh',
                     'Groups' : 'general',
                     'Levels' : '1 2 3 4 5',
                     'Description' : 'A test description' }
        output = dict([ [str[0].strip(), str[1].strip()] for str in [ item.split(':') for item in self.runner.other ]])
        self.assert_(expected == output)

    def testInfoGlob(self):
        "test that globbing works OK with --info"
        self.tg.create_named_test('foobar')
        self.tg.create_named_test('fooblabla')
        self.tg.create_named_test('blabla')

        self.runSectool("-c %s --info foo\*" % self.config_file)
        self.assert_([ tmp[1].strip() for tmp in [ item.split(':') for item in self.runner.other ] if tmp[0].strip() == 'Name'] == [ 'foobar', 'fooblabla'])

    def testInfoGlobAll(self):
        "test that sectool --info \* works OK"
        self.tg.create_named_test('foobar')
        self.tg.create_named_test('fooblabla')
        self.tg.create_named_test('blabla')

        self.runSectool("-c %s --info \*" % self.config_file)
        self.assert_([ tmp[1].strip() for tmp in [ item.split(':') for item in self.runner.other ] if tmp[0].strip() == 'Name'] == [ 'blabla', 'foobar', 'fooblabla'])

class TestAuto():
    def setUp(self):
        pass

    def testNoAutoSection(self):
        pass

    def tearDown(self):
        pass

    def testLevel(self):
        pass

    def testIncludeExclude(self):
        pass

# -------------- run the test suite -------------- #
if __name__ == "__main__":
    unittest.main()
